<?php

declare(strict_types=1);

namespace Drupal\static_web\Form;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure WebStaticSettingsForm settings for this site.
 */
final class WebStaticSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'static_web_web_static_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['static_web.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $uid = \Drupal::currentUser()->id();
    $entity = \Drupal::entityTypeManager()
      ->getStorage('static_web_static_web_users')
      ->loadByProperties(['field_ws_client' => $uid]);
    $entity = !empty($entity) ? reset($entity) : FALSE;

    $form['site'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Domain ID'),
      '#default_value' => $entity ? $entity->field_ws_site_name->value : FALSE,
      '#disabled' => $entity && $entity->field_ws_site_name->value ? TRUE : FALSE,
      '#required' => FALSE,
    ];

    if ($entity) {
      $form['logo'] = [
        '#type' => 'managed_image',
        '#title' => $this->t('Site Logo'),
        '#default_value' => [$entity->field_ws_site_logo->target_id],
        '#upload_validators' => [
          'file_validate_extensions' => ['jpg jpeg png'],
        ],
        '#upload_location' => 'public://ws/',
        '#required' => FALSE,
      ];
    }
    else {
      $form['logo'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Site Logo'),
        '#upload_validators' => [
          'file_validate_extensions' => ['jpg jpeg png'],
        ],
        '#upload_location' => 'public://ws/',
        '#required' => FALSE,
      ];
    }

    $form['pages'] = [
      '#type' => 'details',
      '#title' => $this->t('My Pages'),
      '#open' => TRUE,
      '#attributes' => [
        'id' => 'page_container',
      ],
    ];
    $form['pages']['table'] = $this->buildTable();
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderForm'];
  }

  /**
   * {@inheritdoc}
   */
  protected function getUserPages() {
    $uid = \Drupal::currentUser()->id();
    return \Drupal::entityTypeManager()->getStorage('node')->loadByProperties(['uid' => $uid]);
  }

  /**
   * {@inheritdoc}
   */
  protected function buildTable() {
    $entities = $this->getUserPages();
    $rows = [];
    foreach ($entities as $entity) {
      $rows[$entity->getTitle()] = [
        [
          'data' =>
          new FormattableMarkup('<a href=":link">@name</a>',
          [
            ':link' => '/',
            '@name' => $entity->getTitle(),
          ]),
        ],
        ['data' => $this->checkCurrentHomePage($entity->id())],
        [
          'data' =>
          [
            '#type' => 'submit',
            '#value' => $this->t('Set Homepage'),
            '#submit' => ['::editButtonSubmit'],
          ],
        ],
        [
          'data' =>
          new FormattableMarkup('<a href=":link">@name</a>',
          [
            ':link' => '/node/' . $entity->id() . '/edit',
            '@name' => 'Edit',
          ]),
        ],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => [
        ['data' => 'Title'],
        ['data' => 'Is Homepage ?'],
        ['data' => 'Operation'],
        ['data' => 'Edit'],
      ],
      '#rows' => $rows,
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCurrentHomePage($nid) {
    $uid = \Drupal::currentUser()->id();
    $entity = \Drupal::entityTypeManager()
      ->getStorage('static_web_static_web_users')
      ->loadByProperties(['field_ws_client' => $uid]);
    if (empty($entity)) {
      return FALSE;
    }
    $entity = reset($entity);
    $homepage_nid = $entity->field_ws_home_page->target_id;
    return $nid === $homepage_nid ? $this->t('Yes') : $this->t('No');
  }

  /**
   * {@inheritdoc}
   * */
  protected function generateDomain($site_name) {
    $data['site_name'] = $site_name;
    $data['account_id'] = $this->config('static_web.settings')->get('account');
    $data['email'] = $this->config('static_web.settings')->get('email');
    $data['token'] = $this->config('static_web.settings')->get('token');
    $data['project'] = $this->config('static_web.settings')->get('project');
    $data['project_domain'] = $this->config('static_web.settings')->get('project_domain');
    $data['zone_id'] = $this->config('static_web.settings')->get('zone');
    return \Drupal::service('static_web.domain_handler')->setData($data)->generateDomain();
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $uid = \Drupal::currentUser()->id();
    $domain = $form_state->getValues()['site'];
    $entity = \Drupal::entityTypeManager()
      ->getStorage('static_web_static_web_users')
      ->loadByProperties(['field_ws_site_name' => $domain]);
    if (!empty($entity) && $uid !== reset($entity)->field_ws_client->target_id) {
      return $form_state->setErrorByName('site', $this->t('Domain ID name already used'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $uid = \Drupal::currentUser()->id();
    $entity = \Drupal::entityTypeManager()
      ->getStorage('static_web_static_web_users')
      ->loadByProperties(['field_ws_client' => $uid]);

    if (empty($entity)) {
      $contents = $this->generateDomain($form_state->getValue('site'));
      if (empty($contents)) {
        \Drupal::messenger()->addError('Domain ID is not valid');
      }
      else {
        \Drupal::entityTypeManager()
          ->getStorage('static_web_static_web_users')
          ->create([
            'label' => $form_state->getValue('site'),
            'field_ws_site_name' => strtolower($form_state->getValue('site')),
            'field_ws_site_logo' => $form_state->getValue('logo'),
            'field_ws_client' => $uid,
          ])->save();
      }
    }
    else {
      $entity = reset($entity);
      // Regenerate new domain if changed.
      if ($form_state->getValue('site') !== $entity->field_ws_site_name->value) {
        $this->generateDomain($form_state->getValue('site'));
      }
      $entity->set('label', $form_state->getValue('site'));
      $entity->set('field_ws_site_name', strtolower($form_state->getValue('site')));
      $entity->set('field_ws_site_logo', $form_state->getValue('logo'));
      $entity->save();
    }
    parent::submitForm($form, $form_state);
  }

}
